/*
 * Project name:
     SerialFlash example (Demonstration of usage of the SerialFlash board)
 * Copyright:
     (c) MikroElektronika, 2005-2010.
 * Revision History:
     20100423:
       - initial release;
 * Description:
      Demoinstration of basic reading and writing techniques of SerialFlash
      Initial Command menu let's you choose which command to execute.
      Commands are issued by pressing PORTD buttons.
      0 - Erase chip
      1 - Read Flash ID
      2 - Do a simple writing and reading back a simple byte
      3 - Do a simple writing and reading back a simple byte
 * Test configuration:
     MCU:             PIC18F4520
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39631a.pdf
     Dev.Board:       EasyPIC6
                      http://www.mikroe.com/en/tools/easypic6/
     Oscillator:      HS-PLL, 32.0000 MHz
     Ext. Modules:    LCD 2x16, SerialFlash board
                      ac:SerialFlash_board
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/en/compilers/mikroc/pro/pic/
 * NOTES:
     - Place SerialFlash on PORTC and activate switches 1, 4 and 6
     - Place 2x16 LCD in position
     - Turn on LCD backlight SW2.8 (board specific)
     - Pull-down entire PORTD and place J17 jumper in VCC position
*/

// Lcd pinout settings
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D7 at RB3_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D4 at RB0_bit;

// Pin direction
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D7_Direction at TRISB3_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D4_Direction at TRISB0_bit;

sbit CS_Serial_Flash_bit at LATC0_bit;
sbit CS_Serial_Flash_Direction_bit at TRISC0_bit;

#define _DATA_ARRAY_SIZE 16
#include <built_in.h>
#include <Serial_Flash_driver.h>

unsigned char Refresh_Cmd_Menu = 1;
unsigned char write_array[_DATA_ARRAY_SIZE] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
unsigned char read_array[_DATA_ARRAY_SIZE] = {0};

void main() {
  unsigned char temp, SerialFlashID, txt[12];
  unsigned char i, to_write, success;
  unsigned long address;
  unsigned int half_address;
  CMCON = 0x07;
  ADCON1 = 0x0F;
  TRISD = 0xFF;
  PORTD = 0x00;
  TRISA = 0x00;
  LATA = 0x00;
  // Set SPI1 module to master mode, clock = Fosc/4, data sampled at the middle of interval, 
  // clock idle state low and data transmitted at low to high edge:
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV4, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
  SPI_Set_Active(SPI1_Read, SPI1_Write);
  
  Lcd_Init();
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  Lcd_Out(1, 1, "  SerialFlash   ");
  Lcd_Out(2, 1, "  HW REV 1.00   ");
  Delay_ms(1000);
  
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Out(1, 1, "Initializing... ");
  SerialFlash_init();
  SerialFlash_WriteEnable();
  Delay_ms(500);

  while (1){
    /************************************************************************
    * Erase entire flash
    ************************************************************************/
    if (Button(&PORTD, 0, 1, 1)){
      Lcd_Cmd(_LCD_CLEAR);
      Lcd_Out(1, 1, "Erasing chip... ");
      SerialFlash_ChipErase();
      Refresh_Cmd_Menu = 1;
    }
    /************************************************************************
    * Read Flash ID
    ************************************************************************/
    else if (Button(&PORTD, 1, 1, 1)){
      Lcd_Cmd(_LCD_CLEAR);
      SerialFlashID = SerialFlash_ReadID();
      ByteToStr(SerialFlashID, txt);
      Lcd_Out(1, 1, "Flash ID:       ");
      Lcd_out(1, 11, txt);
      Refresh_Cmd_Menu = 1;
      Delay_ms(1000);
    }
    /************************************************************************
    * Write and read back a single byte
    ************************************************************************/
    else if (Button(&PORTD, 2, 1, 1)){
      Lcd_Cmd(_LCD_CLEAR);
      temp = 221;
      Lcd_Out(1, 1, "Writting:");
      ByteToStr(temp, txt);
      Lcd_out(1, 13, txt);
      SerialFlash_WriteByte(temp, 0x123456);

      Lcd_Out(2, 1, "Reading: ");
      temp = SerialFlash_ReadByte(0x123456);
      ByteToStr(temp, txt);
      Lcd_out(2, 13, txt);
      Refresh_Cmd_Menu = 1;
      Delay_ms(1000);
    }
    /************************************************************************
    * Write the repetitive pattern of data to fill the first 4K of memory
    * and then read the entire flash again and check the data correctness
    ************************************************************************/
    else if (Button(&PORTD, 3, 1, 1)){
      Lcd_Cmd(_LCD_CLEAR);
      Lcd_Out(1, 1, "Writing array...");
      SerialFlash_WriteArray(0x0000, &write_array, _DATA_ARRAY_SIZE);
      Delay_ms(500);
      Lcd_Out(1, 1, "Reading array...");
      SerialFlash_ReadArray(0x0000, &read_array, _DATA_ARRAY_SIZE);
      Delay_ms(500);
      success = 1;
      for (i = 0; i<_DATA_ARRAY_SIZE; i++){
        if (read_array[i] != write_array[i]){
          success = 0;
          break;
        }
      }
      if (success)
        Lcd_Out(2, 1, "Test successfull");
      else
        Lcd_Out(2, 1, "Test failed!    ");
      Delay_ms(1000);
      Refresh_Cmd_Menu = 1;
    }
    /************************************************************************
    * Draw command menu
    ************************************************************************/
    if (Refresh_Cmd_Menu){
      Refresh_Cmd_Menu = 0;
      Lcd_Cmd(_LCD_CLEAR);
      Lcd_Out(1, 1, "ENTER PORTD CMD:");
      Lcd_Out(2, 1, "0-E 1-I 2-RW 3-A");
    }

  }
}