/*
 * Project name:
     SerialFlash example (Demonstration of usage of the SerialFlash board)
 * Copyright:
     (c) MikroElektronika, 2005-2010.
 * Revision History:
     20100423:
       - initial release;
 * Description:
      Demoinstration of basic reading and writing techniques of SerialFlash
      Initial Command menu let's you choose which command to execute.
      Commands are issued by pressing PORTD buttons.
      0 - Erase chip
      1 - Read Flash ID
      2 - Do a simple writing and reading back a simple byte
      3 - Do a simple writing and reading back an array of 16 bytes
 * Test configuration:
     MCU:             dsPIC33FJ128GP710
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70286C.pdf
     Dev.Board:       Lv24-33A
                      http://www.mikroe.com/en/tools/lv24-33a/
     Oscillator:      HS-PLL, 80.0000 MHz
     Ext. Modules:    LCD 2x16, SerialFlash board
                      ac:SerialFlash_board
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/en/compilers/mikroc/pro/dspic/
 * NOTES:
     - Place SerialFlash on PORTGH
     - Place 2x16 LCD in position
     - Turn on LCD backlight SW2.8 (board specific)
     - Pull-down entire PORTD and place J17 jumper in VCC position
*/

// LCD module connections
sbit LCD_RS at LATB2_bit;
sbit LCD_EN at LATB3_bit;
sbit LCD_D4 at LATB4_bit;
sbit LCD_D5 at LATB5_bit;
sbit LCD_D6 at LATB6_bit;
sbit LCD_D7 at LATB7_bit;

sbit LCD_RS_Direction at TRISB2_bit;
sbit LCD_EN_Direction at TRISB3_bit;
sbit LCD_D4_Direction at TRISB4_bit;
sbit LCD_D5_Direction at TRISB5_bit;
sbit LCD_D6_Direction at TRISB6_bit;
sbit LCD_D7_Direction at TRISB7_bit;
// End LCD module connections

sbit CS_Serial_Flash_bit at LATG9_bit;
sbit CS_Serial_Flash_Direction_bit at TRISG9_bit;

static const unsigned short _DATA_ARRAY_SIZE = 16;
#include <built_in.h>
#include <Serial_Flash_driver.h>

unsigned char Refresh_Cmd_Menu = 1;
unsigned char write_array[_DATA_ARRAY_SIZE] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
unsigned char read_array[_DATA_ARRAY_SIZE] = {0};

void main() {
  unsigned char temp, SerialFlashID, txt[12];
  unsigned char i, to_write, success;
  unsigned long address;
  unsigned int half_address;
  ADPCFG = 0xFFFF;
  TRISD = 0xFFFF;
  PORTD = 0x0000;
  // Set SPI1 to the Master Mode, data length is 16-bit,
  // clock = Fcy (no clock scaling), data sampled in the middle of interval,
  // clock IDLE state high and data transmitted at low to high clock edge:
  SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_2, _SPI_PRESCALE_PRI_1,
    _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_END, _SPI_CLK_IDLE_LOW, _SPI_IDLE_2_ACTIVE);

  SPI_Set_Active(SPI2_Read, SPI2_Write);
  
  Lcd_Init();
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  Lcd_Out(1, 1, "  SerialFlash   ");
  Lcd_Out(2, 1, "  HW REV 1.00   ");
  Delay_ms(1000);
  
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Out(1, 1, "Initializing... ");
  SerialFlash_init();
  SerialFlash_WriteEnable();
  Delay_ms(500);

  while (1){
    /************************************************************************
    * Erase entire flash
    ************************************************************************/
    if (Button(&PORTD, 0, 1, 1)){
      Lcd_Cmd(_LCD_CLEAR);
      Lcd_Out(1, 1, "Erasing chip... ");
      SerialFlash_ChipErase();
      Refresh_Cmd_Menu = 1;
    }
    /************************************************************************
    * Read Flash ID
    ************************************************************************/
    else if (Button(&PORTD, 1, 1, 1)){
      Lcd_Cmd(_LCD_CLEAR);
      SerialFlashID = SerialFlash_ReadID();
      ByteToStr(SerialFlashID, txt);
      Lcd_Out(1, 1, "Flash ID:       ");
      Lcd_out(1, 11, txt);
      Refresh_Cmd_Menu = 1;
      Delay_ms(1000);
    }
    /************************************************************************
    * Write and read back a single byte
    ************************************************************************/
    else if (Button(&PORTD, 2, 1, 1)){
      Lcd_Cmd(_LCD_CLEAR);
      temp = 221;
      Lcd_Out(1, 1, "Writting:");
      ByteToStr(temp, txt);
      Lcd_out(1, 13, txt);
      SerialFlash_WriteByte(temp, 0x123456);

      Lcd_Out(2, 1, "Reading: ");
      temp = SerialFlash_ReadByte(0x123456);
      ByteToStr(temp, txt);
      Lcd_out(2, 13, txt);
      Refresh_Cmd_Menu = 1;
      Delay_ms(1000);
    }
    /************************************************************************
    * Write the repetitive pattern of data to fill the first 4K of memory
    * and then read the entire flash again and check the data correctness
    ************************************************************************/
    else if (Button(&PORTD, 3, 1, 1)){
      Lcd_Cmd(_LCD_CLEAR);
      Lcd_Out(1, 1, "Writing array...");
      SerialFlash_WriteArray(0x0000, &write_array, _DATA_ARRAY_SIZE);
      Delay_ms(500);
      Lcd_Out(1, 1, "Reading array...");
      SerialFlash_ReadArray(0x0000, &read_array, _DATA_ARRAY_SIZE);
      Delay_ms(500);
      success = 1;
      for (i = 0; i<_DATA_ARRAY_SIZE; i++){
        if (read_array[i] != write_array[i]){
          success = 0;
          break;
        }
      }
      if (success)
        Lcd_Out(2, 1, "Test successfull");
      else
        Lcd_Out(2, 1, "Test failed!    ");
      Delay_ms(1000);
      Refresh_Cmd_Menu = 1;
    }
    /************************************************************************
    * Draw command menu
    ************************************************************************/
    if (Refresh_Cmd_Menu){
      Refresh_Cmd_Menu = 0;
      Lcd_Cmd(_LCD_CLEAR);
      Lcd_Out(1, 1, "ENTER PORTD CMD:");
      Lcd_Out(2, 1, "0-E 1-I 2-RW 3-A");
    }

  }
}